import shutil
import subprocess

from pants.backend.jvm.tasks.nailgun_task import NailgunTask
from pants.base.exceptions import TaskError
from pants.base.workunit import WorkUnitLabel
from pants.java.jar.jar_dependency import JarDependency
from pants.task.simple_codegen_task import SimpleCodegenTask

from caustic.pants.targets.caustic_library import CausticLibrary


class CausticGen(SimpleCodegenTask, NailgunTask):
    """
    Generates Scala source code from Caustic *.acid files.
    """

    @classmethod
    def register_options(cls, register):
        super(CausticGen, cls).register_options(register)
        print 'HELLO'
        print self.options_scope
        cls.register_jvm_tool(register, 'causticc', classpath_spec='//:causticc', classpath=[
            JarDependency(org='com.madavan', name='caustic-compiler_2.12', rev='2.0.3')
        ])

    def is_gentarget(self, target):
        return isinstance(target, CausticLibrary)

    def synthetic_target_type(self, target):
        return CausticLibrary

    def synthetic_target_extra_dependencies(self, target, target_workdir):
        return self.resolve_deps([
            'caustic-library/src/main/scala',
            'caustic-runtime/src/main/scala',
            '3rdparty/jvm:spray-json',
        ])

    def execute_codegen(self, target, target_workdir):
        # Generate Scala code using the Caustic compiler.
        sources = target.target_base
        main = "caustic.compiler.Causticc"
        classpath = self.tool_classpath('causticc')
        result = self.runjava(classpath=classpath, main=main, args=sources, workunit_name='compile')

        if result != 0:
            raise TaskError('Causticc ... exited non-zero ({})'.format(result))

        # Move all the generated sources to the target_workdir.
        for src in target.sources_relative_to_buildroot():
            gen = src.replace('.acid', '.scala')
            shutil.move(gen, target_workdir)



from pants.backend.jvm.targets.jvm_target import JvmTarget


class CausticLibrary(JvmTarget):
    """
    A Caustic library generated from Caustic source files.
    """

    def __init__(self, **kwargs):
        super(CausticLibrary, self).__init__(**kwargs)



from pants.build_graph.build_file_aliases import BuildFileAliases
from pants.goal.task_registrar import TaskRegistrar as task

from caustic.pants.targets.caustic_library import CausticLibrary
from caustic.pants.tasks.caustic_gen import CausticGen


def build_file_aliases():
    return BuildFileAliases(
        targets={
            'caustic_library': CausticLibrary,
        },
    )


def register_goals():
    task(name='causticc', action=CausticGen).install('gen')




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: PythonLibrary(BuildFileAddress(caustic-compiler/src/main/pants/BUILD, pants))

from setuptools import setup

setup(**
{   'author': 'Ashwin Madavan',
    'author_email': 'ashwin.madavan@gmail.com',
    'classifiers': [   'Intended Audience :: Developers',
                       'License :: OSI Approved :: Apache Software License',
                       'Operating System :: OS Independent',
                       'Programming Language :: Python'],
    'description': 'Caustic plugin for Pants',
    'install_requires': [],
    'license': 'Apache License, Version 2.0',
    'name': 'caustic.pants',
    'package_data': {   },
    'package_dir': {   '': 'src'},
    'packages': [   'caustic',
                    'caustic.pants',
                    'caustic.pants.targets',
                    'caustic.pants.tasks'],
    'url': 'https://github.com/ashwin153/caustic',
    'version': '1.0.6',
    'zip_safe': True}
)
